#!/bin/sh
#
# check_disks - Nagios plugin, reports used disk space
# by Michael Tosch

#set +f needed
# Posix
PATH=/usr/xpg4/bin:/bin:/usr/bin:/usr/sbin:/sbin
export PATH

for mtab in /etc/mnttab /etc/mtab /proc/mounts
do
  [ -r $mtab ] && break
done
if [ ! -r $mtab ]
then
  mtab=/tmp/mtab.$$
  trap "rm -f $mtab" 0 HUP INT SYS
  mount | awk '$1~/^\//' > $mtab
fi

checkint(){
  case $1 in
  *[!0-9]*|"") echo "$0: '$1' is not an integer"; exit 3;;
  esac
}

print_usage(){
  echo "Usage:
$0
  -c <integer>    returns CRITICAL if above <integer> % full
  -w <integer>    returns WARNING if above <integer> % full and not CRITICAL
 [-o]             OS disks (usually all mountpoints of the root fs type)
 [-d <disk>] ...  mountpoint or device (wildcards allowed)
 [-r ERE]         include mountpoints that match the ExtendedRegularExpression
 [-x <disk>] ...  exclude mountpoint or device"
}

# Getting parameters
while getopts "od:r:x:w:c:h" OPT
do
  case $OPT in
  o) disks="$disks
`
# exclude /boot and /stand, include /tmp, skip identical devices (bind-mounts)
awk '$2=="/" {type=$3} ($3==type && $2!="/boot" && $2!="/stand" || $2=="/tmp") && d[$1]++==0 {print $2}' $mtab
`";;
  r) disks="$disks
`awk '($2~o) {print $2}' o="$OPTARG" $mtab`";;
  d) disks="$disks
$OPTARG";;
  w) warn=$OPTARG;;
  c) crit=$OPTARG;;
  x) xdisks="$xdisks
$OPTARG";;
  h)
    echo "This plugin shows the % used disk space.
"
    print_usage; exit 3;;
  *) print_usage; exit 3;;
  esac
done

# check thresholds
if [ -z "$warn" -o -z "$crit" ]; then
  print_usage; exit 3
fi

checkint "$warn"
checkint "$crit"

# eliminate duplicates and xdisks
if [ -n "$xdisks" ]; then
 disks=`echo "$disks" | awk '(NF>0 && s[$1]++==0)' | fgrep -vx "$xdisks"`
else
 disks=`echo "$disks" | awk '(NF>0 && s[$1]++==0)'`
fi

df -kP $disks 2>&1 |
awk -v w="$warn" -v c="$crit" '
# df error e.g. disk not found
($1=="df:" && NF>2) {err=err FS $0; next}
# eliminate header and duplicates
(++nr>1 && NF>=6 && s[$1 FS $6]++==0) {
 # auto scale
 if ($2<1048576) {scale=1024; unit="MB"} else
 if ($2<1073741824) {scale=1048576; unit="GB"} else
 if ($2<1099511627776) {scale=1073741824; unit="TB"} else
  {scale=1099511627776; unit="PB"}
 if ($5+0>w) {
  if ($5+0>max) {max=$5+0}
  # problem disks first
  msg=sprintf (" %s %.1f %s (%s);",$6,$3/scale,unit,$5) msg
 } else {
  msg=msg sprintf (" %s %.1f %s (%s);",$6,$3/scale,unit,$5)
 }
 total=$3+$4 # $2 can be higher
 graph=graph sprintf (" %s=%.1f%s;%.1f;%.1f;0;%.1f",$6,$3/scale,unit,total/100*w/scale,total/100*c/scale,total/scale)
}
END {
 if (err!="") {print "CRITICAL:" err "|" graph; exit 2}
 if (max>c) {state="CRITICAL"; x=2} else if (max>w) {state="WARNING"; x=1} else {state="OK"; x=0}
 print state " - used space:" msg "|" graph
 exit x
}'

